package com.energy.online.data;

// 新增Spring Boot相关导入

import com.alibaba.fastjson2.JSON;
import com.energy.online.data.dto.BaseConfig;
import com.energy.online.data.dto.Line;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.scheduling.annotation.EnableAsync;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;
import wei.yigulu.iec104.apdumodel.Apdu;
import wei.yigulu.iec104.apdumodel.Asdu;
import wei.yigulu.iec104.asdudataframe.PulseTotalSummonType;
import wei.yigulu.iec104.nettyconfig.Iec104MasterBuilder;
import wei.yigulu.iec104.util.SendDataFrameHelper;

import javax.annotation.PostConstruct;
import java.util.List;

@SpringBootApplication
@EnableAsync
@EnableScheduling
public class OnlineDataMain {

    @Autowired
    private BaseConfig baseConfig;

    @Qualifier("taskExecutor")
    @Autowired
    private ThreadPoolTaskExecutor taskExecutor;

    @PostConstruct
    public void init() {
        List<Line> lines = baseConfig.getLines();
        for (Line line : lines) {
            taskExecutor.execute(() -> runDeviceConnection(line));
        }
    }

    private void runDeviceConnection(Line line) {
        try {
            Iec104MasterBuilder master = new Iec104MasterBuilder(line.getIp(), line.getPort());
            master.createByUnBlock();

            while (!Thread.currentThread().isInterrupted()) {
                System.out.println(JSON.toJSONString(line));
                Thread.sleep(3000L);
                SendDataFrameHelper.sendTotalSummonFrame(master.getFuture().channel(), 10, 6, master.getLog());
                PulseTotalSummonType pulseTotalSummonType = new PulseTotalSummonType();
                Asdu asdu = pulseTotalSummonType.generateBack();
                Apdu apdu = new Apdu();
                apdu.setAsdu(asdu);
                master.sendFrameToOpposite(apdu.encode());
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }


    public static void main(String[] args) {
        SpringApplication.run(OnlineDataMain.class, args);
    }

}
